@@ -55,22 +55,62 @@ void stun_debug_disable (void) {
   debug_enabled = 0;
 }
 
+#if     __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
+#define GNUC_PRINTF(format_idx, arg_idx) \
+  __attribute__((__format__ (__printf__, format_idx, arg_idx)))
+#else
+#define GNUC_PRINTF( format_idx, arg_idx)
+#endif
+
+static void
+default_handler (const char *format, va_list ap) GNUC_PRINTF (1, 0);
+
+static void
+default_handler (const char *format, va_list ap)
+{
+  vfprintf (stderr, format, ap);
+  fprintf (stderr, "\n");
+}
+
+static StunDebugHandler handler = default_handler;
+
 void stun_debug (const char *fmt, ...)
 {
   va_list ap;
   if (debug_enabled) {
     va_start (ap, fmt);
-    vfprintf (stderr, fmt, ap);
+    handler (fmt, ap);
     va_end (ap);
   }
 }
 
-void stun_debug_bytes (const void *data, size_t len)
+void stun_debug_bytes (const char *prefix, const void *data, size_t len)
 {
   size_t i;
+  size_t prefix_len = strlen (prefix);
+  char *bytes;
+
+  if (!debug_enabled)
+    return;
+
+  bytes = malloc (prefix_len + 2 + (len * 2) + 1);
+  bytes[0] = 0;
+  strcpy (bytes, prefix);
+  strcpy (bytes + prefix_len, "0x");
 
-  stun_debug ("0x");
   for (i = 0; i < len; i++)
-    stun_debug ("%02x", ((const unsigned char *)data)[i]);
+    sprintf (bytes + prefix_len + 2 + (i * 2), "%02x", ((const unsigned char *)data)[i]);
+
+  stun_debug ("%s", bytes);
+  free (bytes);
+}
+
+
+void stun_set_debug_handler (StunDebugHandler _handler)
+{
+  if (_handler == NULL)
+    _handler = default_handler;
+
+  handler = _handler;
 }
 
