@@ -54,6 +54,6 @@ typedef struct {
 } crc_data;
 
 
-uint32_t crc32 (const crc_data *data, size_t n, bool wlm2009_stupid_crc32_typo);
+uint32_t stun_crc32 (const crc_data *data, size_t n, bool wlm2009_stupid_crc32_typo);
 
 #endif /* _CRC32_H */
